@@ -7,14 +7,15 @@ Primarily supports `$ rio insp`.
 
 from __future__ import absolute_import
 
+from collections import OrderedDict
 import logging
 import warnings
 
 import numpy as np
 
 import rasterio
-from rasterio._io import RasterReader
-
+from rasterio.io import DatasetReader
+from rasterio.transform import guard_transform
 from rasterio.compat import zip_longest
 
 logger = logging.getLogger(__name__)
@@ -34,16 +35,14 @@ def get_plt():
 
 
 def show(source, with_bounds=True, contour=False, contour_label_kws=None,
-         ax=None, title=None, **kwargs):
+         ax=None, title=None, transform=None, adjust='linear', **kwargs):
     """Display a raster or raster band using matplotlib.
 
     Parameters
     ----------
-    source : array-like in raster axis order,
-        or (raster dataset, bidx) tuple,
-        or raster dataset,
-        If the tuple (raster dataset, bidx),
-        selects band `bidx` from raster.  If raster dataset display the rgb image
+    source : array or dataset object opened in 'r' mode or Band or tuple(dataset, bidx)
+        If Band or tuple (dataset, bidx), display the selected band.
+        If raster dataset display the rgb image
         as defined in the colorinterp metadata, or default to first band.
     with_bounds : bool (opt)
         Whether to change the image extent to the spatial bounds of the image,
@@ -58,6 +57,13 @@ def show(source, with_bounds=True, contour=False, contour_label_kws=None,
         Axis to plot on, otherwise uses current axis.
     title : str, optional
         Title for the figure.
+    transform : Affine, optional
+        Defines the affine transform if source is an array
+    adjust : 'linear' | None
+        If the plotted data is an RGB image, adjust the values of
+        each band so that they fall between 0 and 1 before plotting. If
+        'linear', values will be adjusted by the min / max of each band. If
+        None, no adjustment will be applied.
     **kwargs : key, value pairings optional
         These will be passed to the matplotlib imshow or contour method
         depending on contour argument.
@@ -75,22 +81,25 @@ def show(source, with_bounds=True, contour=False, contour_label_kws=None,
 
     if isinstance(source, tuple):
         arr = source[0].read(source[1])
+        if len(arr.shape) >= 3:
+            arr = reshape_as_image(arr)
         if with_bounds:
             kwargs['extent'] = plotting_extent(source[0])
-    elif isinstance(source, RasterReader):
+    elif isinstance(source, DatasetReader):
+        if with_bounds:
+            kwargs['extent'] = plotting_extent(source)
+
         if source.count == 1:
             arr = source.read(1, masked=True)
         else:
             try:
-                source_colorinterp = {source.colorinterp(n): n for n in source.indexes}
+                source_colorinterp = OrderedDict(zip(source.indexes, source.colorinterp))
                 colorinterp = rasterio.enums.ColorInterp
                 rgb_indexes = [source_colorinterp[ci] for ci in
                                (colorinterp.red, colorinterp.green, colorinterp.blue)]
                 arr = source.read(rgb_indexes, masked=True)
                 arr = reshape_as_image(arr)
 
-                if with_bounds:
-                    kwargs['extent'] = plotting_extent(source)
             except KeyError:
                 arr = source.read(1, masked=True)
     else:
@@ -100,6 +109,14 @@ def show(source, with_bounds=True, contour=False, contour_label_kws=None,
             arr = reshape_as_image(source)
         else:
             arr = source
+        if transform and with_bounds:
+            kwargs['extent'] = plotting_extent(arr, transform)
+    if adjust is True and arr.ndim >= 3:
+        # Adjust each band by the min/max so it will plot as RGB.
+        arr = reshape_as_raster(arr)
+        for ii, band in enumerate(arr):
+            arr[ii] = adjust_band(band, kind='linear')
+        arr = reshape_as_image(arr)
 
     show = False
     if not ax:
@@ -130,17 +147,36 @@ def show(source, with_bounds=True, contour=False, contour_label_kws=None,
     return ax
 
 
-def plotting_extent(source):
+def plotting_extent(source, transform=None):
     """Returns an extent in the format needed
      for matplotlib's imshow (left, right, bottom, top)
      instead of rasterio's bounds (left, bottom, top, right)
 
     Parameters
     ----------
-    source : raster dataset
+    source : array or dataset object opened in 'r' mode
+        input data
+    transform: Affine, required if source is array
+        Defines the affine transform if source is an array
+
+    Returns
+    -------
+    tuple of float
+        left, right, bottom, top
     """
-    extent = (source.bounds.left, source.bounds.right,
-              source.bounds.bottom, source.bounds.top)
+    if hasattr(source, 'bounds'):
+        extent = (source.bounds.left, source.bounds.right,
+                  source.bounds.bottom, source.bounds.top)
+    elif not transform:
+        raise ValueError(
+            "transform is required if source is an array")
+    else:
+        transform = guard_transform(transform)
+        rows, cols = source.shape[0:2]
+        left, top = transform * (0, 0)
+        right, bottom = transform * (cols, rows)
+        extent = (left, right, bottom, top)
+
     return extent
 
 
@@ -153,14 +189,14 @@ def reshape_as_image(arr):
 
     Parameters
     ----------
-    source : array-like in a of format (bands, rows, columns)
+    arr : array-like of shape (bands, rows, columns)
+        image to reshape
     """
     # swap the axes order from (bands, rows, columns) to (rows, columns, bands)
-    im = np.ma.transpose(arr, [1,2,0])
+    im = np.ma.transpose(arr, [1, 2, 0])
     return im
 
 
-
 def reshape_as_raster(arr):
     """Returns the array in a raster order
     by swapping the axes order from (rows, columns, bands)
@@ -169,9 +205,10 @@ def reshape_as_raster(arr):
     Parameters
     ----------
     arr : array-like in the image form of (rows, columns, bands)
+        image to reshape
     """
     # swap the axes order from (rows, columns, bands) to (bands, rows, columns)
-    im = np.transpose(arr, [2,0,1])
+    im = np.transpose(arr, [2, 0, 1])
     return im
 
 
@@ -180,8 +217,9 @@ def show_hist(source, bins=10, masked=True, title='Histogram', ax=None, **kwargs
 
     Parameters
     ----------
-    source : np.array or RasterReader, rasterio.Band or tuple(dataset, bidx)
-        Input data to display.  The first three arrays in multi-dimensional
+    source : array or dataset object opened in 'r' mode or Band or tuple(dataset, bidx)
+        Input data to display.
+        The first three arrays in multi-dimensional
         arrays are plotted as red, green, and blue.
     bins : int, optional
         Compute histogram across N bins.
@@ -198,7 +236,7 @@ def show_hist(source, bins=10, masked=True, title='Histogram', ax=None, **kwargs
     """
     plt = get_plt()
 
-    if isinstance(source, RasterReader):
+    if isinstance(source, DatasetReader):
         arr = source.read(masked=masked)
     elif isinstance(source, (tuple, rasterio.Band)):
         arr = source[0].read(source[1], masked=masked)
@@ -207,7 +245,7 @@ def show_hist(source, bins=10, masked=True, title='Histogram', ax=None, **kwargs
 
     # The histogram is computed individually for each 'band' in the array
     # so we need the overall min/max to constrain the plot
-    rng = arr.min(), arr.max()
+    rng = np.nanmin(arr), np.nanmax(arr)
 
     if len(arr.shape) is 2:
         arr = np.expand_dims(arr.flatten(), 0).T
@@ -216,7 +254,7 @@ def show_hist(source, bins=10, masked=True, title='Histogram', ax=None, **kwargs
         arr = arr.reshape(arr.shape[0], -1).T
         colors = ['red', 'green', 'blue', 'violet', 'gold', 'saddlebrown']
 
-    #The goal is to provide a curated set of colors for working with
+    # The goal is to provide a curated set of colors for working with
     # smaller datasets and let matplotlib define additional colors when
     # working with larger datasets.
     if arr.shape[-1] > len(colors):
@@ -241,11 +279,11 @@ def show_hist(source, bins=10, masked=True, title='Histogram', ax=None, **kwargs
     fig = ax.get_figure()
 
     ax.hist(arr,
-             bins=bins,
-             color=colors,
-             label=labels,
-             range=rng,
-             **kwargs)
+            bins=bins,
+            color=colors,
+            label=labels,
+            range=rng,
+            **kwargs)
 
     ax.legend(loc="upper right")
     ax.set_title(title, fontweight='bold')
@@ -254,3 +292,28 @@ def show_hist(source, bins=10, masked=True, title='Histogram', ax=None, **kwargs
     ax.set_ylabel('Frequency')
     if show:
         plt.show()
+
+
+def adjust_band(band, kind='linear'):
+    """Adjust a band to be between 0 and 1.
+
+    Parameters
+    ----------
+    band : array, shape (height, width)
+        A band of a raster object.
+    kind : 'linear'
+        The kind of normalization to apply. For now, there
+        is only one option ('linear').
+
+    Returns
+    -------
+    band_normed : array, shape (height, width)
+        An adjusted version of the input band.
+    """
+    band_normed = np.zeros_like(band)
+    if kind == 'linear':
+        # Including this `if` statement in case future norms are added.
+        imin = np.nanmin(band)
+        imax = np.nanmax(band)
+        band_normed = (band - imin) / (imax - imin)
+    return band_normed
