@@ -1,11 +1,17 @@
+"""Unittests for $ rio warp"""
+
+
 import logging
 import os
 import sys
 
+import affine
 import numpy as np
 import pytest
 
 import rasterio
+from rasterio.env import GDALVersion
+from rasterio.warp import SUPPORTED_RESAMPLING, GDAL2_RESAMPLING
 from rasterio.rio import warp
 from rasterio.rio.main import main_group
 
@@ -20,12 +26,9 @@ def test_dst_crs_error(runner, tmpdir):
     result = runner.invoke(main_group, [
         'warp', srcname, outputname, '--dst-crs', '{foo: bar}'])
     assert result.exit_code == 2
-    assert 'for dst_crs: crs appears to be JSON but is not' in result.output
+    assert 'for dst_crs: CRS appears to be JSON but is not' in result.output
 
 
-@pytest.mark.xfail(
-    os.environ.get('GDALVERSION', 'a.b.c').startswith('1.9'),
-                   reason="GDAL 1.9 doesn't catch this error")
 def test_dst_crs_error_2(runner, tmpdir):
     """Invalid PROJ.4 is a bad parameter."""
     srcname = 'tests/data/RGB.byte.tif'
@@ -33,7 +36,7 @@ def test_dst_crs_error_2(runner, tmpdir):
     result = runner.invoke(main_group, [
         'warp', srcname, outputname, '--dst-crs', '{"proj": "foobar"}'])
     assert result.exit_code == 2
-    assert 'for dst_crs: Failed to initialize PROJ.4' in result.output
+    assert 'for dst_crs: Invalid' in result.output
 
 
 def test_dst_crs_error_epsg(runner, tmpdir):
@@ -114,7 +117,7 @@ def test_warp_no_reproject(runner, tmpdir):
     """ When called without parameters, output should be same as source """
     srcname = 'tests/data/shade.tif'
     outputname = str(tmpdir.join('test.tif'))
-    result = runner.invoke(warp.warp, [srcname, outputname])
+    result = runner.invoke(main_group, ['warp', srcname, outputname])
     assert result.exit_code == 0
     assert os.path.exists(outputname)
 
@@ -124,15 +127,15 @@ def test_warp_no_reproject(runner, tmpdir):
             assert output.crs == src.crs
             assert output.nodata == src.nodata
             assert np.allclose(output.bounds, src.bounds)
-            assert output.affine.almost_equals(src.affine)
+            assert output.transform.almost_equals(src.transform)
             assert np.allclose(output.read(1), src.read(1))
 
 
 def test_warp_no_reproject_dimensions(runner, tmpdir):
     srcname = 'tests/data/shade.tif'
     outputname = str(tmpdir.join('test.tif'))
-    result = runner.invoke(warp.warp, [srcname, outputname,
-                                       '--dimensions', '100', '100'])
+    result = runner.invoke(main_group, [
+        'warp', srcname, outputname, '--dimensions', '100', '100'])
     assert result.exit_code == 0
     assert os.path.exists(outputname)
 
@@ -142,21 +145,21 @@ def test_warp_no_reproject_dimensions(runner, tmpdir):
             assert output.width == 100
             assert output.height == 100
             assert np.allclose([97.839396, 97.839396],
-                                  [output.affine.a, -output.affine.e])
+                               [output.transform.a, -output.transform.e])
 
 
 def test_warp_no_reproject_res(runner, tmpdir):
     srcname = 'tests/data/shade.tif'
     outputname = str(tmpdir.join('test.tif'))
-    result = runner.invoke(warp.warp, [srcname, outputname,
-                                       '--res', 30])
+    result = runner.invoke(main_group, [
+        'warp', srcname, outputname, '--res', 30])
     assert result.exit_code == 0
     assert os.path.exists(outputname)
 
     with rasterio.open(srcname) as src:
         with rasterio.open(outputname) as output:
             assert output.crs == src.crs
-            assert np.allclose([30, 30], [output.affine.a, -output.affine.e])
+            assert np.allclose([30, 30], [output.transform.a, -output.transform.e])
             assert output.width == 327
             assert output.height == 327
 
@@ -165,8 +168,8 @@ def test_warp_no_reproject_bounds(runner, tmpdir):
     srcname = 'tests/data/shade.tif'
     outputname = str(tmpdir.join('test.tif'))
     out_bounds = [-11850000, 4810000, -11849000, 4812000]
-    result = runner.invoke(warp.warp,[srcname, outputname,
-                                      '--bounds'] + out_bounds)
+    result = runner.invoke(main_group, [
+        'warp', srcname, outputname, '--bounds'] + out_bounds)
     assert result.exit_code == 0
     assert os.path.exists(outputname)
 
@@ -174,8 +177,8 @@ def test_warp_no_reproject_bounds(runner, tmpdir):
         with rasterio.open(outputname) as output:
             assert output.crs == src.crs
             assert np.allclose(output.bounds, out_bounds)
-            assert np.allclose([src.affine.a, src.affine.e],
-                                  [output.affine.a, output.affine.e])
+            assert np.allclose([src.transform.a, src.transform.e],
+                               [output.transform.a, output.transform.e])
             assert output.width == 105
             assert output.height == 210
 
@@ -184,9 +187,8 @@ def test_warp_no_reproject_bounds_res(runner, tmpdir):
     srcname = 'tests/data/shade.tif'
     outputname = str(tmpdir.join('test.tif'))
     out_bounds = [-11850000, 4810000, -11849000, 4812000]
-    result = runner.invoke(warp.warp,[srcname, outputname,
-                                      '--res', 30,
-                                      '--bounds', ] + out_bounds)
+    result = runner.invoke(main_group, [
+        'warp', srcname, outputname, '--res', 30, '--bounds'] + out_bounds)
     assert result.exit_code == 0
     assert os.path.exists(outputname)
 
@@ -194,16 +196,49 @@ def test_warp_no_reproject_bounds_res(runner, tmpdir):
         with rasterio.open(outputname) as output:
             assert output.crs == src.crs
             assert np.allclose(output.bounds, out_bounds)
-            assert np.allclose([30, 30], [output.affine.a, -output.affine.e])
+            assert np.allclose([30, 30], [output.transform.a, -output.transform.e])
             assert output.width == 34
             assert output.height == 67
 
+    # dst-bounds should be an alias to bounds
+    outputname = str(tmpdir.join('test2.tif'))
+    out_bounds = [-11850000, 4810000, -11849000, 4812000]
+    result = runner.invoke(main_group, [
+        'warp', srcname, outputname, '--res', 30, '--dst-bounds'] + out_bounds)
+    assert result.exit_code == 0
+    assert os.path.exists(outputname)
+    with rasterio.open(srcname) as src:
+        with rasterio.open(outputname) as output:
+            assert np.allclose(output.bounds, out_bounds)
+
+
+def test_warp_no_reproject_src_bounds_dimensions(runner, tmpdir):
+    """--src-bounds option works with dimensions"""
+    srcname = 'tests/data/shade.tif'
+    outputname = str(tmpdir.join('test.tif'))
+    out_bounds = [-11850000, 4810000, -11849000, 4812000]
+    result = runner.invoke(
+        main_group, [
+            'warp', srcname, outputname, '--dimensions', 9, 14,
+            '--src-bounds'] + out_bounds)
+    assert result.exit_code == 0
+    assert os.path.exists(outputname)
+
+    with rasterio.open(srcname) as src:
+        with rasterio.open(outputname) as output:
+            assert output.crs == src.crs
+            assert np.allclose(output.bounds, out_bounds)
+            assert np.allclose([111.111111, 142.857142],
+                               [output.transform.a, -output.transform.e])
+            assert output.width == 9
+            assert output.height == 14
+
 
 def test_warp_reproject_dst_crs(runner, tmpdir):
     srcname = 'tests/data/RGB.byte.tif'
     outputname = str(tmpdir.join('test.tif'))
-    result = runner.invoke(warp.warp, [srcname, outputname,
-                                       '--dst-crs', 'EPSG:4326'])
+    result = runner.invoke(main_group, [
+        'warp', srcname, outputname, '--dst-crs', 'EPSG:4326'])
     assert result.exit_code == 0
     assert os.path.exists(outputname)
 
@@ -213,17 +248,17 @@ def test_warp_reproject_dst_crs(runner, tmpdir):
             assert output.crs == {'init': 'epsg:4326'}
             assert output.width == 835
             assert output.height == 696
-            assert np.allclose(output.bounds,
-                                  [-78.95864996545055, 23.564787976164418,
-                                   -76.5759177302349, 25.550873767433984])
+            assert np.allclose(output.bounds, [
+                -78.95864996545055, 23.564787976164418,
+                -76.5759177302349, 25.550873767433984])
 
 
 def test_warp_reproject_dst_crs_proj4(runner, tmpdir):
     proj4 = '+proj=longlat +ellps=WGS84 +datum=WGS84'
     srcname = 'tests/data/shade.tif'
     outputname = str(tmpdir.join('test.tif'))
-    result = runner.invoke(warp.warp, [srcname, outputname,
-                                       '--dst-crs', proj4])
+    result = runner.invoke(main_group, [
+        'warp', srcname, outputname, '--dst-crs', proj4])
     assert result.exit_code == 0
     assert os.path.exists(outputname)
 
@@ -234,15 +269,14 @@ def test_warp_reproject_dst_crs_proj4(runner, tmpdir):
 def test_warp_reproject_res(runner, tmpdir):
     srcname = 'tests/data/shade.tif'
     outputname = str(tmpdir.join('test.tif'))
-    result = runner.invoke(warp.warp, [srcname, outputname,
-                                       '--dst-crs', 'EPSG:4326',
-                                       '--res', 0.01])
+    result = runner.invoke(main_group, [
+        'warp', srcname, outputname, '--dst-crs', 'EPSG:4326', '--res', 0.01])
     assert result.exit_code == 0
     assert os.path.exists(outputname)
 
     with rasterio.open(outputname) as output:
         assert output.crs == {'init': 'epsg:4326'}
-        assert np.allclose([0.01, 0.01], [output.affine.a, -output.affine.e])
+        assert np.allclose([0.01, 0.01], [output.transform.a, -output.transform.e])
         assert output.width == 9
         assert output.height == 7
 
@@ -250,40 +284,56 @@ def test_warp_reproject_res(runner, tmpdir):
 def test_warp_reproject_dimensions(runner, tmpdir):
     srcname = 'tests/data/shade.tif'
     outputname = str(tmpdir.join('test.tif'))
-    result = runner.invoke(warp.warp, [srcname, outputname,
-                                       '--dst-crs', 'EPSG:4326',
-                                       '--dimensions', '100', '100'])
+    result = runner.invoke(main_group, [
+        'warp', srcname, outputname, '--dst-crs', 'EPSG:4326',
+        '--dimensions', '100', '100'])
     assert result.exit_code == 0
     assert os.path.exists(outputname)
 
-    with rasterio.open(srcname) as src:
-        with rasterio.open(outputname) as output:
-            assert output.crs == {'init': 'epsg:4326'}
-            assert output.width == 100
-            assert output.height == 100
-            assert np.allclose([0.0008789062498762235, 0.0006771676143921468],
-                                  [output.affine.a, -output.affine.e])
+    with rasterio.open(outputname) as output:
+        assert output.crs == {'init': 'epsg:4326'}
+        assert output.width == 100
+        assert output.height == 100
+        assert np.allclose([0.0008789062498762235, 0.0006771676143921468],
+                           [output.transform.a, -output.transform.e])
+
+
+def test_warp_reproject_dimensions_invalid_params(runner, tmpdir):
+    srcname = 'tests/data/shade.tif'
+    outputname = str(tmpdir.join('test.tif'))
+    bad_params = [
+        ['--bounds', '0', '0', '10', '10'],
+        ['--res', '10']
+    ]
+
+    for param in bad_params:
+        result = runner.invoke(warp.warp,
+                               [srcname, outputname, '--dst-crs', 'EPSG:4326',
+                                '--dimensions', '100', '100'] +
+                               param)
+
+        assert result.exit_code == 2
+        assert '--dimensions cannot be used with' in result.output
 
 
 def test_warp_reproject_bounds_no_res(runner, tmpdir):
     srcname = 'tests/data/shade.tif'
     outputname = str(tmpdir.join('test.tif'))
     out_bounds = [-11850000, 4810000, -11849000, 4812000]
-    result = runner.invoke(warp.warp, [srcname, outputname,
-                                       '--dst-crs', 'EPSG:4326',
-                                       '--bounds', ] + out_bounds)
+    result = runner.invoke(main_group, [
+        'warp', srcname, outputname, '--dst-crs', 'EPSG:4326', '--bounds'] +
+        out_bounds)
     assert result.exit_code == 2
 
 
 def test_warp_reproject_multi_bounds_fail(runner, tmpdir):
-    """Mixing --bounds and --x-dst-bounds fails."""
+    """Mixing --bounds and --src-bounds fails."""
     srcname = 'tests/data/shade.tif'
     outputname = str(tmpdir.join('test.tif'))
     out_bounds = [-11850000, 4810000, -11849000, 4812000]
-    result = runner.invoke(warp.warp, [srcname, outputname,
-                                       '--dst-crs', 'EPSG:4326',
-                                       '--x-dst-bounds'] + out_bounds +
-                                       ['--bounds'] + out_bounds)
+    result = runner.invoke(main_group, [
+        'warp', srcname, outputname, '--dst-crs', 'EPSG:4326',
+        '--src-bounds'] + out_bounds + ['--bounds'] + out_bounds)
     assert result.exit_code == 2
 
 
@@ -292,79 +342,88 @@ def test_warp_reproject_bounds_crossup_fail(runner, tmpdir):
     srcname = 'tests/data/shade.tif'
     outputname = str(tmpdir.join('test.tif'))
     out_bounds = [-11850000, 4810000, -11849000, 4812000]
-    result = runner.invoke(warp.warp, [srcname, outputname,
-                                       '--dst-crs', 'EPSG:4326',
-                                       '--res', 0.001, '--x-dst-bounds', ]
-                                       + out_bounds)
+    result = runner.invoke(main_group, [
+        'warp', srcname, outputname, '--dst-crs', 'EPSG:4326', '--res', 0.001,
+        '--bounds'] + out_bounds)
     assert result.exit_code == 2
 
 
-def test_warp_reproject_bounds_res_future_warning(runner, tmpdir):
-    """Use of --bounds results in a warning from the 1.0 future."""
-    srcname = 'tests/data/shade.tif'
-    outputname = str(tmpdir.join('test.tif'))
-    out_bounds = [-11850000, 4810000, -11849000, 4812000]
-    result = runner.invoke(
-                warp.warp, [srcname, outputname, '--dst-crs', 'EPSG:4326',
-                            '--res', 0.001, '--bounds'] + out_bounds)
-    assert "Future Warning" in result.output
-
-
 def test_warp_reproject_src_bounds_res(runner, tmpdir):
     """--src-bounds option works."""
     srcname = 'tests/data/shade.tif'
     outputname = str(tmpdir.join('test.tif'))
     out_bounds = [-11850000, 4810000, -11849000, 4812000]
     result = runner.invoke(
-        warp.warp, [srcname, outputname, '--dst-crs', 'EPSG:4326',
-                    '--res', 0.001, '--src-bounds'] + out_bounds)
+        main_group, [
+            'warp', srcname, outputname, '--dst-crs', 'EPSG:4326',
+            '--res', 0.001, '--src-bounds'] + out_bounds)
     assert result.exit_code == 0
     assert os.path.exists(outputname)
 
-    with rasterio.open(srcname) as src:
-        with rasterio.open(outputname) as output:
-            assert output.crs == {'init': 'epsg:4326'}
-            assert np.allclose(output.bounds[:],
-                                  [-106.45036, 39.6138, -106.44136, 39.6278])
-            assert np.allclose([0.001, 0.001],
-                                  [output.affine.a, -output.affine.e])
-            assert output.width == 9
-            assert output.height == 14
+    with rasterio.open(outputname) as output:
+        assert output.crs == {'init': 'epsg:4326'}
+        assert np.allclose(output.bounds[:],
+                           [-106.45036, 39.6138, -106.44136, 39.6278])
+        assert np.allclose([0.001, 0.001],
+                           [output.transform.a, -output.transform.e])
+        assert output.width == 9
+        assert output.height == 14
+
+
+def test_warp_reproject_src_bounds_dimensions(runner, tmpdir):
+    """--src-bounds option works with dimensions"""
+    srcname = 'tests/data/shade.tif'
+    outputname = str(tmpdir.join('test.tif'))
+    out_bounds = [-11850000, 4810000, -11849000, 4812000]
+    result = runner.invoke(
+        main_group, [
+            'warp', srcname, outputname, '--dst-crs', 'EPSG:4326',
+            '--dimensions', 9, 14, '--src-bounds'] + out_bounds)
+    assert result.exit_code == 0
+    assert os.path.exists(outputname)
+
+    with rasterio.open(outputname) as output:
+        assert output.crs == {'init': 'epsg:4326'}
+        assert np.allclose(output.bounds[:],
+                           [-106.45036, 39.6138, -106.44136, 39.6278])
+        assert round(output.transform.a, 4) == 0.001
+        assert round(-output.transform.e, 4) == 0.001
 
 
 def test_warp_reproject_dst_bounds(runner, tmpdir):
-    """--x-dst-bounds option works."""
+    """--bounds option works."""
     srcname = 'tests/data/shade.tif'
     outputname = str(tmpdir.join('test.tif'))
     out_bounds = [-106.45036, 39.6138, -106.44136, 39.6278]
     result = runner.invoke(
-        warp.warp, [srcname, outputname, '--dst-crs', 'EPSG:4326',
-                    '--res', 0.001, '--x-dst-bounds'] + out_bounds)
+        main_group, [
+            'warp', srcname, outputname, '--dst-crs', 'EPSG:4326',
+            '--res', 0.001, '--bounds'] + out_bounds)
     assert result.exit_code == 0
     assert os.path.exists(outputname)
 
-    with rasterio.open(srcname) as src:
-        with rasterio.open(outputname) as output:
-            assert output.crs == {'init': 'epsg:4326'}
-            assert np.allclose(output.bounds[0::3],
-                                  [-106.45036, 39.6278])
-            assert np.allclose([0.001, 0.001],
-                                  [output.affine.a, -output.affine.e])
+    with rasterio.open(outputname) as output:
+        assert output.crs == {'init': 'epsg:4326'}
+        assert np.allclose(output.bounds[0::3],
+                           [-106.45036, 39.6278])
+        assert np.allclose([0.001, 0.001],
+                           [output.transform.a, -output.transform.e])
 
-            # XXX: an extra row and column is produced in the dataset
-            # because we're using ceil instead of floor internally.
-            # Not necessarily a bug, but may change in the future.
-            assert np.allclose([output.bounds[2]-0.001, output.bounds[1]+0.001],
-                                  [-106.44136, 39.6138])
-            assert output.width == 10
-            assert output.height == 15
+        # XXX: an extra row and column is produced in the dataset
+        # because we're using ceil instead of floor internally.
+        # Not necessarily a bug, but may change in the future.
+        assert np.allclose([output.bounds[2] - 0.001, output.bounds[1] + 0.001],
+                           [-106.44136, 39.6138])
+        assert output.width == 10
+        assert output.height == 15
 
 
 def test_warp_reproject_like(runner, tmpdir):
     likename = str(tmpdir.join('like.tif'))
     kwargs = {
         "crs": {'init': 'epsg:4326'},
-        "transform": (-106.523, 0.001, 0, 39.6395, 0, -0.001),
+        "transform": affine.Affine(0.001, 0, -106.523,
+                                   0, -0.001, 39.6395),
         "count": 1,
         "dtype": rasterio.uint8,
         "driver": "GTiff",
@@ -373,30 +432,49 @@ def test_warp_reproject_like(runner, tmpdir):
         "nodata": 0
     }
 
-    with rasterio.Env():
-        with rasterio.open(likename, 'w', **kwargs) as dst:
-            data = np.zeros((10, 10), dtype=rasterio.uint8)
-            dst.write(data, indexes=1)
+    with rasterio.open(likename, 'w', **kwargs) as dst:
+        data = np.zeros((10, 10), dtype=rasterio.uint8)
+        dst.write(data, indexes=1)
 
     srcname = 'tests/data/shade.tif'
     outputname = str(tmpdir.join('test.tif'))
-    result = runner.invoke(warp.warp, [srcname, outputname,
-                                       '--like', likename])
+    result = runner.invoke(main_group, [
+        'warp', srcname, outputname, '--like', likename])
     assert result.exit_code == 0
     assert os.path.exists(outputname)
 
     with rasterio.open(outputname) as output:
         assert output.crs == {'init': 'epsg:4326'}
-        assert np.allclose([0.001, 0.001], [output.affine.a, -output.affine.e])
+        assert np.allclose(
+            [0.001, 0.001], [output.transform.a, -output.transform.e])
         assert output.width == 10
         assert output.height == 10
 
 
+def test_warp_reproject_like_invalid_params(runner, tmpdir):
+    srcname = 'tests/data/shade.tif'
+    outputname = str(tmpdir.join('test.tif'))
+    bad_params = [
+        ['--dimensions', '10', '10'],
+        ['--dst-crs', 'EPSG:4326'],
+        ['--bounds', '0', '0', '10', '10'],
+        ['--res', '10']
+    ]
+
+    for param in bad_params:
+        result = runner.invoke(warp.warp,
+                               [srcname, outputname, '--like', srcname] +
+                               param)
+
+        assert result.exit_code == 2
+        assert '--like cannot be used with any of' in result.output
+
+
 def test_warp_reproject_nolostdata(runner, tmpdir):
     srcname = 'tests/data/world.byte.tif'
     outputname = str(tmpdir.join('test.tif'))
-    result = runner.invoke(warp.warp, [srcname, outputname,
-                                       '--dst-crs', 'EPSG:3857'])
+    result = runner.invoke(main_group, [
+        'warp', srcname, outputname, '--dst-crs', 'EPSG:3857'])
     assert result.exit_code == 0
     assert os.path.exists(outputname)
 
@@ -417,10 +495,8 @@ def test_warp_dst_crs_empty_string(runner, tmpdir):
     infile = 'tests/data/RGB.byte.tif'
     outfile = str(tmpdir.mkdir('empty_warp_dst_crs.tif').join('test.tif'))
 
-    result = runner.invoke(warp.warp, [
-        infile,
-        outfile,
-        '--dst-crs', ''])
+    result = runner.invoke(main_group, [
+        'warp', infile, outfile, '--dst-crs', ''])
 
     assert result.exit_code != 0
     assert 'empty or invalid' in result.output
@@ -429,9 +505,9 @@ def test_warp_dst_crs_empty_string(runner, tmpdir):
 def test_warp_badcrs_dimensions(runner, tmpdir):
     srcname = 'tests/data/shade.tif'
     outputname = str(tmpdir.join('test.tif'))
-    result = runner.invoke(warp.warp, [srcname, outputname,
-                                       '--dst-crs', '{"init": "epsg:-1"}',
-                                       '--dimensions', '100', '100'])
+    result = runner.invoke(main_group, [
+        'warp', srcname, outputname, '--dst-crs', '{"init": "epsg:-1"}',
+        '--dimensions', '100', '100'])
     assert result.exit_code == 2
     assert "Invalid value for dst_crs" in result.output
 
@@ -441,33 +517,94 @@ def test_warp_badcrs_src_bounds(runner, tmpdir):
     outputname = str(tmpdir.join('test.tif'))
     out_bounds = [-11850000, 4810000, -11849000, 4812000]
     result = runner.invoke(
-        warp.warp, [srcname, outputname,
-                    '--dst-crs', '{"init": "epsg:-1"}',
-                    '--res', 0.001, '--src-bounds'] + out_bounds)
+        main_group, [
+            'warp', srcname, outputname, '--dst-crs', '{"init": "epsg:-1"}',
+            '--res', 0.001, '--src-bounds'] + out_bounds)
     assert result.exit_code == 2
     assert "Invalid value for dst_crs" in result.output
 
 
-def test_warp_reproject_check_invert(runner, tmpdir):
-    srcname = 'tests/data/world.rgb.tif'
+def test_warp_reproject_check_invert_true(runner, tmpdir):
     outputname = str(tmpdir.join('test.tif'))
-    result = runner.invoke(warp.warp, [srcname, outputname,
-                                       '--check-invert-proj', 'yes',
-                                       '--dst-crs', 'EPSG:3759'])
-    assert result.exit_code == 0
-    assert os.path.exists(outputname)
-
-    with rasterio.open(outputname) as output:
-        assert output.crs == {'init': 'epsg:3759'}
-        shape1 = output.shape
-
     output2name = str(tmpdir.join('test2.tif'))
-    result = runner.invoke(warp.warp, [srcname, output2name,
-                                       '--check-invert-proj', 'no',
-                                       '--dst-crs', 'EPSG:3759'])
-    assert result.exit_code == 0
-    assert os.path.exists(output2name)
+    srcname = 'tests/data/world.rgb.tif'
 
-    with rasterio.open(output2name) as output:
-        assert output.crs == {'init': 'epsg:3759'}
-        assert output.shape != shape1
+    # default True
+    runner.invoke(main_group, [
+        'warp', srcname, outputname, '--dst-crs', 'EPSG:3759'])
+
+    # explicit True
+    runner.invoke(main_group, [
+        'warp', srcname, output2name, '--check-invert-proj',
+        '--dst-crs', 'EPSG:3759'])
+
+    with rasterio.open(outputname) as output, rasterio.open(output2name) as output2:
+        assert output.shape == output2.shape
+
+
+def test_warp_reproject_check_invert_false(runner, tmpdir):
+    outputname = str(tmpdir.join('test.tif'))
+    output2name = str(tmpdir.join('test2.tif'))
+    srcname = 'tests/data/world.rgb.tif'
+
+    # default True
+    runner.invoke(main_group, [
+        'warp', srcname, outputname, '--dst-crs', 'EPSG:3759'])
+
+    # explicit False
+    runner.invoke(main_group, [
+        'warp', srcname, output2name, '--no-check-invert-proj',
+        '--dst-crs', 'EPSG:3759'])
+
+    with rasterio.open(outputname) as output, rasterio.open(output2name) as output2:
+        assert output.shape != output2.shape
+
+
+def test_warp_vrt_gcps(runner, tmpdir):
+    """A VRT with GCPs can be warped."""
+    srcname = 'tests/data/white-gemini-iv.vrt'
+    outputname = str(tmpdir.join('test.tif'))
+    result = runner.invoke(main_group, [
+        'warp', srcname, outputname, '--dst-crs', 'epsg:32618'])
+    assert result.exit_code == 0
+
+    # All 4 corners have no data.
+    with rasterio.open(outputname) as src:
+        data = src.read()
+        assert not data[:, 0, 0].any()
+        assert not data[:, 0, -1].any()
+        assert not data[:, -1, -1].any()
+        assert not data[:, -1, 0].any()
+
+
+@pytest.mark.parametrize("method", SUPPORTED_RESAMPLING)
+def test_warp_resampling(runner, path_rgb_byte_tif, tmpdir, method):
+    """Resampling methods supported by this version of GDAL should run
+    successfully"""
+
+    outputname = str(tmpdir.join('test.tif'))
+    result = runner.invoke(main_group, [
+        'warp', path_rgb_byte_tif, outputname,
+        '--dst-crs', 'epsg:3857',
+        '--resampling', method.name])
+
+    print(result.output)
+    assert result.exit_code == 0
+
+
+@pytest.mark.skipif(
+    GDALVersion.runtime().at_least('2.0'),
+    reason="Test only applicable to GDAL < 2.0")
+@pytest.mark.parametrize("method", GDAL2_RESAMPLING)
+def test_warp_resampling_not_yet_supported(
+        runner, path_rgb_byte_tif, tmpdir, method):
+    """Resampling methods not yet supported should fail with error"""
+
+    outputname = str(tmpdir.join('test.tif'))
+    result = runner.invoke(main_group, [
+        'warp', path_rgb_byte_tif, outputname,
+        '--dst-crs', 'epsg:3857',
+        '--resampling', method.name])
+
+    assert result.exit_code == 2
+    assert 'Invalid value for "--resampling"' in result.output
